# Generated by Django 3.2.2 on 2021-06-19 06:28

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("snitch", "0002_alter_eventtype_verb"),
    ]

    operations = [
        migrations.AlterField(
            model_name="eventtype",
            name="verb",
            field=models.CharField(max_length=255, null=True, unique=True),
        ),
    ]
