# Generated by Django 2.2.7 on 2019-11-18 15:40

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("schedules", "0001_initial")]

    operations = [
        migrations.AlterField(
            model_name="schedule",
            name="verb",
            field=models.CharField(
                choices=[
                    ("activated", "activated"),
                    ("confirmed", "confirmed"),
                    ("dummy", "dummy"),
                    ("every hour", "every hour"),
                ],
                max_length=255,
                null=True,
                verbose_name="verb",
            ),
        )
    ]
